/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { useEffect, useMemo } from 'react';
import { METRIC_TYPE, UiCounterMetricType } from '@kbn/analytics';
import { UsageCollectionSetup } from '@kbn/usage-collection-plugin/public';
import { useKibana } from '@kbn/kibana-react-plugin/public';
import { USAGE_COLLECTION_APP_NAME } from '../../../common/constants';

/**
 * Note: The usage_collection plugin will take care of sending this data to the telemetry server.
 * You can find the metrics that are collected by these hooks in Stack Telemetry.
 * Search the index `kibana-ui-counter`. You can filter for `eventName` and/or `appName`.
 */

interface TrackOptions {
  metricType?: UiCounterMetricType;
  delay?: number; // in ms
}
type EffectDeps = unknown[];

interface ServiceDeps {
  usageCollection: UsageCollectionSetup; // TODO: This should really be start. Looking into it.
}

export type TrackMetricOptions = TrackOptions & { metric: string };
export type UiTracker = ReturnType<typeof useUiTracker>;
export type TrackEvent = (options: TrackMetricOptions) => void;

export { METRIC_TYPE };

export function useUiTracker<Services extends ServiceDeps>(): TrackEvent {
  const reportUiCounter = useKibana<Services>().services?.usageCollection?.reportUiCounter;
  const trackEvent = useMemo(() => {
    return ({ metric, metricType = METRIC_TYPE.COUNT }: TrackMetricOptions) => {
      if (reportUiCounter) {
        reportUiCounter(USAGE_COLLECTION_APP_NAME, metricType, metric);
      }
    };
  }, [reportUiCounter]);
  return trackEvent;
}

export function useTrackMetric<Services extends ServiceDeps>(
  { metric, metricType = METRIC_TYPE.COUNT, delay = 0 }: TrackMetricOptions,
  effectDependencies: EffectDeps = []
) {
  const reportUiCounter = useKibana<Services>().services?.usageCollection?.reportUiCounter;

  useEffect(() => {
    if (!reportUiCounter) {
      // eslint-disable-next-line no-console
      console.log(
        'usageCollection.reportUiCounter is unavailable. Ensure this is setup via <KibanaContextProvider />.'
      );
    } else {
      let decoratedMetric = metric;
      if (delay > 0) {
        decoratedMetric += `__delayed_${delay}ms`;
      }
      const id = setTimeout(
        () => reportUiCounter(USAGE_COLLECTION_APP_NAME, metricType, decoratedMetric),
        Math.max(delay, 0)
      );
      return () => clearTimeout(id);
    }
    // the dependencies are managed externally
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, effectDependencies);
}

/**
 * useTrackPageview is a convenience wrapper for tracking a pageview
 * Its metrics will be found at:
 * stack_stats.kibana.plugins.ui_metric.{app}.pageview__{path}(__delayed_{n}ms)?
 */
type TrackPageviewProps = TrackOptions & { path: string };

export function useTrackPageview<Services extends ServiceDeps>(
  { path, ...rest }: TrackPageviewProps,
  effectDependencies: EffectDeps = []
) {
  useTrackMetric<Services>({ ...rest, metric: `pageview__${path}` }, effectDependencies);
}
