/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { ThreadPoolSearchRejectionsRule } from './thread_pool_search_rejections_rule';
import { RULE_THREAD_POOL_SEARCH_REJECTIONS } from '../../common/constants';
import { fetchThreadPoolRejectionStats } from '../lib/alerts/fetch_thread_pool_rejections_stats';
import { fetchClusters } from '../lib/alerts/fetch_clusters';
import { alertsMock } from '@kbn/alerting-plugin/server/mocks';
import { ALERT_REASON } from '@kbn/rule-data-utils';

const RealDate = Date;

jest.mock('../lib/alerts/fetch_thread_pool_rejections_stats', () => ({
  fetchThreadPoolRejectionStats: jest.fn(),
}));
jest.mock('../lib/alerts/fetch_clusters', () => ({
  fetchClusters: jest.fn(),
}));

jest.mock('../static_globals', () => ({
  Globals: {
    app: {
      getLogger: () => ({ debug: jest.fn() }),
      url: 'http://localhost:5601',
      config: {
        ui: {
          show_license_expiration: true,
          ccs: { enabled: true },
          container: { elasticsearch: { enabled: false } },
        },
      },
    },
  },
}));

describe('ThreadpoolSearchRejectionsRule', () => {
  it('should have defaults', () => {
    const rule = new ThreadPoolSearchRejectionsRule();
    expect(rule.ruleOptions.id).toBe(RULE_THREAD_POOL_SEARCH_REJECTIONS);
    expect(rule.ruleOptions.name).toBe('Thread pool search rejections');
    expect(rule.ruleOptions.throttle).toBe('1d');
    expect(rule.ruleOptions.defaultParams).toStrictEqual({ threshold: 300, duration: '5m' });
    expect(rule.ruleOptions.actionVariables).toStrictEqual([
      { name: 'node', description: 'The node reporting high thread pool search rejections.' },
      {
        name: 'internalShortMessage',
        description: 'The short internal message generated by Elastic.',
      },
      {
        name: 'internalFullMessage',
        description: 'The full internal message generated by Elastic.',
      },
      { name: 'state', description: 'The current state of the alert.' },
      { name: 'clusterName', description: 'The cluster to which the node(s) belongs.' },
      { name: 'action', description: 'The recommended action for this alert.' },
      {
        name: 'actionPlain',
        description: 'The recommended action for this alert, without any markdown.',
      },
    ]);
  });
  describe('execute', () => {
    function FakeDate() {}
    FakeDate.prototype.valueOf = () => 1;

    const clusterUuid = 'abc123';
    const clusterName = 'testCluster';
    const nodeId = 'esNode1';
    const nodeName = 'esName1';
    const threadPoolType = 'search';
    const rejectionCount = 400;
    const stat = [
      {
        rejectionCount,
        type: threadPoolType,
        clusterUuid,
        nodeId,
        nodeName,
        ccs: null,
      },
    ];

    const services = alertsMock.createRuleExecutorServices();
    const executorOptions = { services, state: {} };

    beforeEach(() => {
      // @ts-ignore
      Date = FakeDate;
      (fetchThreadPoolRejectionStats as jest.Mock).mockImplementation(() => {
        return stat;
      });
      (fetchClusters as jest.Mock).mockImplementation(() => {
        return [{ clusterUuid, clusterName }];
      });
    });

    afterEach(() => {
      Date = RealDate;
      jest.resetAllMocks();
    });

    it('should fire action', async () => {
      const rule = new ThreadPoolSearchRejectionsRule();
      const type = rule.getRuleType();
      await type.executor({
        ...executorOptions,
        params: rule.ruleOptions.defaultParams,
      } as any);
      expect(services.alertsClient.report).toHaveBeenCalledTimes(1);
      expect(services.alertsClient.setAlertData).toHaveBeenCalledTimes(1);
      expect(services.alertsClient.report).toHaveBeenCalledWith({
        id: 'esNode1',
        actionGroup: 'default',
        state: {
          alertStates: [
            {
              ccs: null,
              cluster: { clusterUuid, clusterName },
              nodeId,
              nodeName,
              itemLabel: undefined,
              meta: {
                rejectionCount,
                clusterUuid,
                type: threadPoolType,
                nodeId,
                nodeName,
                ccs: null,
              },
              ui: {
                isFiring: true,
                message: {
                  text: `Node #start_link${nodeName}#end_link is reporting ${rejectionCount} ${threadPoolType} rejections at #absolute`,
                  nextSteps: [
                    {
                      text: '#start_linkMonitor this node#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'link',
                          url: 'elasticsearch/nodes/esNode1/advanced',
                        },
                      ],
                    },
                    {
                      text: '#start_linkOptimize complex queries#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}blog/advanced-tuning-finding-and-fixing-slow-elasticsearch-queries',
                        },
                      ],
                    },
                    {
                      text: '#start_linkAdd more nodes#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}guide/en/elasticsearch/reference/{docLinkVersion}/add-elasticsearch-nodes.html',
                        },
                      ],
                    },
                    {
                      text: '#start_linkResize your deployment (ECE)#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}guide/en/cloud-enterprise/current/ece-resize-deployment.html',
                        },
                      ],
                    },
                    {
                      text: '#start_linkThread pool settings#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}guide/en/elasticsearch/reference/{docLinkVersion}/modules-threadpool.html',
                        },
                      ],
                    },
                  ],
                  tokens: [
                    {
                      startToken: '#absolute',
                      type: 'time',
                      isAbsolute: true,
                      isRelative: false,
                      timestamp: 1,
                    },
                    {
                      startToken: '#start_link',
                      endToken: '#end_link',
                      type: 'link',
                      url: `elasticsearch/nodes/${nodeId}`,
                    },
                  ],
                },
                severity: 'danger',
                triggeredMS: 1,
                lastCheckedMS: 0,
              },
            },
          ],
        },
      });
      expect(services.alertsClient.setAlertData).toHaveBeenCalledWith({
        id: 'esNode1',
        context: {
          internalFullMessage: `Thread pool search rejections alert is firing for node ${nodeName} in cluster: ${clusterName}. [View node](http://localhost:5601/app/monitoring#/elasticsearch/nodes/${nodeId}?_g=(cluster_uuid:'${clusterUuid}'))`,
          internalShortMessage: `Thread pool search rejections alert is firing for node ${nodeName} in cluster: ${clusterName}. Verify thread pool ${threadPoolType} rejections for the affected node.`,
          node: `${nodeName}`,
          action: `[View node](http://localhost:5601/app/monitoring#/elasticsearch/nodes/${nodeId}?_g=(cluster_uuid:'${clusterUuid}'))`,
          actionPlain: `Verify thread pool ${threadPoolType} rejections for the affected node.`,
          clusterName,
          count: 1,
          threadPoolType,
          state: 'firing',
        },
        payload: {
          [ALERT_REASON]: `Thread pool search rejections alert is firing for node ${nodeName} in cluster: ${clusterName}. Verify thread pool ${threadPoolType} rejections for the affected node.`,
        },
      });
    });
    it('should not fire actions if under threshold', async () => {
      (fetchThreadPoolRejectionStats as jest.Mock).mockImplementation(() => {
        return [
          {
            ...stat[0],
            rejectionCount: 1,
          },
        ];
      });
      const rule = new ThreadPoolSearchRejectionsRule();
      const type = rule.getRuleType();
      await type.executor({
        ...executorOptions,
        params: rule.ruleOptions.defaultParams,
      } as any);
      expect(services.alertsClient.report).not.toHaveBeenCalled();
      expect(services.alertsClient.setAlertData).not.toHaveBeenCalled();
    });

    it('should handle ccs', async () => {
      const ccs = 'testCluster';
      (fetchThreadPoolRejectionStats as jest.Mock).mockImplementation(() => {
        return [
          {
            ...stat[0],
            ccs,
          },
        ];
      });
      const rule = new ThreadPoolSearchRejectionsRule();
      const type = rule.getRuleType();
      await type.executor({
        ...executorOptions,
        params: rule.ruleOptions.defaultParams,
      } as any);
      const count = 1;
      expect(services.alertsClient.report).toHaveBeenCalledTimes(1);
      expect(services.alertsClient.setAlertData).toHaveBeenCalledTimes(1);
      expect(services.alertsClient.report).toHaveBeenCalledWith({
        id: 'esNode1',
        actionGroup: 'default',
        state: {
          alertStates: [
            {
              ccs: 'testCluster',
              cluster: { clusterUuid, clusterName },
              nodeId,
              nodeName,
              itemLabel: undefined,
              meta: {
                rejectionCount,
                clusterUuid,
                type: threadPoolType,
                nodeId,
                nodeName,
                ccs: 'testCluster',
              },
              ui: {
                isFiring: true,
                message: {
                  text: `Node #start_link${nodeName}#end_link is reporting ${rejectionCount} ${threadPoolType} rejections at #absolute`,
                  nextSteps: [
                    {
                      text: '#start_linkMonitor this node#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'link',
                          url: 'elasticsearch/nodes/esNode1/advanced',
                        },
                      ],
                    },
                    {
                      text: '#start_linkOptimize complex queries#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}blog/advanced-tuning-finding-and-fixing-slow-elasticsearch-queries',
                        },
                      ],
                    },
                    {
                      text: '#start_linkAdd more nodes#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}guide/en/elasticsearch/reference/{docLinkVersion}/add-elasticsearch-nodes.html',
                        },
                      ],
                    },
                    {
                      text: '#start_linkResize your deployment (ECE)#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}guide/en/cloud-enterprise/current/ece-resize-deployment.html',
                        },
                      ],
                    },
                    {
                      text: '#start_linkThread pool settings#end_link',
                      tokens: [
                        {
                          startToken: '#start_link',
                          endToken: '#end_link',
                          type: 'docLink',
                          partialUrl:
                            '{elasticWebsiteUrl}guide/en/elasticsearch/reference/{docLinkVersion}/modules-threadpool.html',
                        },
                      ],
                    },
                  ],
                  tokens: [
                    {
                      startToken: '#absolute',
                      type: 'time',
                      isAbsolute: true,
                      isRelative: false,
                      timestamp: 1,
                    },
                    {
                      startToken: '#start_link',
                      endToken: '#end_link',
                      type: 'link',
                      url: `elasticsearch/nodes/${nodeId}`,
                    },
                  ],
                },
                severity: 'danger',
                triggeredMS: 1,
                lastCheckedMS: 0,
              },
            },
          ],
        },
      });
      expect(services.alertsClient.setAlertData).toHaveBeenCalledWith({
        id: 'esNode1',
        context: {
          internalFullMessage: `Thread pool search rejections alert is firing for node ${nodeName} in cluster: ${clusterName}. [View node](http://localhost:5601/app/monitoring#/elasticsearch/nodes/${nodeId}?_g=(cluster_uuid:'${clusterUuid}',ccs:${ccs}))`,
          internalShortMessage: `Thread pool search rejections alert is firing for node ${nodeName} in cluster: ${clusterName}. Verify thread pool ${threadPoolType} rejections for the affected node.`,
          node: `${nodeName}`,
          action: `[View node](http://localhost:5601/app/monitoring#/elasticsearch/nodes/esNode1?_g=(cluster_uuid:'${clusterUuid}',ccs:testCluster))`,
          actionPlain: `Verify thread pool ${threadPoolType} rejections for the affected node.`,
          clusterName,
          count,
          state: 'firing',
          threadPoolType,
        },
        payload: {
          [ALERT_REASON]: `Thread pool search rejections alert is firing for node ${nodeName} in cluster: ${clusterName}. Verify thread pool ${threadPoolType} rejections for the affected node.`,
        },
      });
    });
  });
});
