/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { useState, useCallback, useEffect, useMemo } from 'react';
import { i18n } from '@kbn/i18n';

import { getDefaultFleetServerpolicyId } from '../../../../../../common/services/agent_policies_helpers';
import type { useComboInput, useInput, useSwitchInput } from '../../../hooks';
import {
  sendCreateAgentPolicy,
  sendGetOneAgentPolicy,
  useFleetStatus,
  useStartServices,
} from '../../../hooks';
import type { NewAgentPolicy } from '../../../types';
import type { FleetServerHost } from '../../../types';
import { useServiceToken } from '../../../hooks/use_service_token';

import { useSelectFleetServerPolicy } from './use_select_fleet_server_policy';
import { useFleetServerHost } from './use_fleet_server_host';

function getQuickStartFleetServerPolicyFields(spaceId?: string): NewAgentPolicy {
  return {
    id: getDefaultFleetServerpolicyId(spaceId),
    name: 'Fleet Server Policy',
    description: 'Fleet Server policy generated by Kibana',
    namespace: 'default',
    has_fleet_server: true,
    monitoring_enabled: ['logs', 'metrics'],
    is_default_fleet_server: true,
  };
}

export type QuickStartCreateFormStatus = 'initial' | 'loading' | 'error' | 'success';

export interface QuickStartCreateForm {
  status: QuickStartCreateFormStatus;
  fleetServerHosts: FleetServerHost[];
  error?: string;
  submit: () => void;
  setFleetServerHost: React.Dispatch<React.SetStateAction<FleetServerHost | undefined | null>>;
  fleetServerHost?: FleetServerHost | null;
  isFleetServerHostSubmitted: boolean;
  fleetServerPolicyId?: string;
  serviceToken?: string;
  inputs: {
    hostUrlsInput: ReturnType<typeof useComboInput>;
    nameInput: ReturnType<typeof useInput>;
    isDefaultInput: ReturnType<typeof useSwitchInput>;
  };
  onClose?: () => void;
}

/**
 * Provides a unified interface that combines the following operations:
 * 1. Setting a Fleet Server host in Fleet's settings
 * 2. Creating an agent policy that contains the `fleet_server` integration
 * 3. Generating a service token used by Fleet Server
 */
export const useQuickStartCreateForm = (): QuickStartCreateForm => {
  const [status, setStatus] = useState<'initial' | 'loading' | 'error' | 'success'>('initial');
  const [error, setError] = useState<string | undefined>();

  const {
    fleetServerHosts,
    fleetServerHost,
    isFleetServerHostSubmitted,
    handleSubmitForm,
    error: fleetServerError,
    setFleetServerHost,
    inputs,
  } = useFleetServerHost();
  const { spaceId } = useFleetStatus();

  // When a validation error is surfaced from the Fleet Server host form, we want to treat it
  // the same way we do errors from the service token or policy creation steps
  useEffect(() => {
    setStatus('error');
    setError(fleetServerError);
  }, [fleetServerError]);

  const { notifications } = useStartServices();
  const { fleetServerPolicyId, setFleetServerPolicyId } = useSelectFleetServerPolicy();
  const { serviceToken, generateServiceToken } = useServiceToken();

  const quickStartFleetServerPolicyFields = useMemo(
    () => getQuickStartFleetServerPolicyFields(spaceId),
    [spaceId]
  );

  const submit = useCallback(async () => {
    try {
      if (!fleetServerHost || fleetServerHost) {
        setStatus('loading');

        if (!fleetServerHost) {
          const res = await handleSubmitForm();
          if (res) {
            setFleetServerHost(res);
          } else {
            setStatus('initial');
            return;
          }
        }

        await generateServiceToken();

        const existingPolicy = await sendGetOneAgentPolicy(quickStartFleetServerPolicyFields.id!);

        // Don't attempt to create the policy if it's already been created in a previous quick start flow
        if (existingPolicy.data?.item) {
          setFleetServerPolicyId(existingPolicy.data?.item.id);
        } else {
          const createPolicyResponse = await sendCreateAgentPolicy(
            quickStartFleetServerPolicyFields,
            {
              withSysMonitoring: true,
            }
          );
          setFleetServerPolicyId(createPolicyResponse.data?.item.id);
        }

        setStatus('success');
      }
    } catch (err) {
      notifications.toasts.addError(err, {
        title: i18n.translate('xpack.fleet.fleetServerSetup.errorAddingFleetServerHostTitle', {
          defaultMessage: 'Error adding Fleet Server host',
        }),
      });

      setStatus('error');
      setError(err.message);
    }
  }, [
    fleetServerHost,
    handleSubmitForm,
    setFleetServerHost,
    generateServiceToken,
    setFleetServerPolicyId,
    notifications.toasts,
    quickStartFleetServerPolicyFields,
  ]);

  return {
    status,
    error,
    submit,
    fleetServerPolicyId,
    fleetServerHosts,
    fleetServerHost,
    setFleetServerHost,
    isFleetServerHostSubmitted,
    serviceToken,
    inputs,
  };
};
