/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */
import { dump, load } from 'js-yaml';

import { ElasticsearchAssetType } from '../../../../types';
import type { RegistryDataStream } from '../../../../types';
import { getPathParts } from '../../archive';

import { getPipelineNameForDatastream } from '../../../../../common/services';

import type { PipelineInstall, RewriteSubstitution } from './types';

export const isTopLevelPipeline = (path: string) => {
  const pathParts = getPathParts(path);
  return (
    pathParts.type === ElasticsearchAssetType.ingestPipeline && pathParts.dataset === undefined
  );
};

export const getPipelineNameForInstallation = ({
  pipelineName,
  dataStream,
  packageVersion,
}: {
  pipelineName: string;
  dataStream?: RegistryDataStream;
  packageVersion: string;
}): string => {
  if (dataStream !== undefined) {
    const isPipelineEntry = pipelineName === dataStream.ingest_pipeline;
    const suffix = isPipelineEntry ? '' : `-${pipelineName}`;
    // if this is the pipeline entry, don't add a suffix
    return `${getPipelineNameForDatastream({ dataStream, packageVersion })}${suffix}`;
  }
  // It's a top-level pipeline
  return `${packageVersion}-${pipelineName}`;
};

export function rewriteIngestPipeline(
  pipeline: string,
  substitutions: RewriteSubstitution[]
): string {
  substitutions.forEach((sub) => {
    const { source, target, templateFunction } = sub;
    // This fakes the use of the golang text/template expression {{SomeTemplateFunction 'some-param'}}
    // cf. https://github.com/elastic/beats/blob/master/filebeat/fileset/fileset.go#L294

    // "Standard style" uses '{{' and '}}' as delimiters
    const matchStandardStyle = `{{\\s?${templateFunction}\\s+['"]${source}['"]\\s?}}`;
    // "Beats style" uses '{<' and '>}' as delimiters because this is current practice in the beats project
    const matchBeatsStyle = `{<\\s?${templateFunction}\\s+['"]${source}['"]\\s?>}`;

    const regexStandardStyle = new RegExp(matchStandardStyle);
    const regexBeatsStyle = new RegExp(matchBeatsStyle);
    pipeline = pipeline.replace(regexStandardStyle, target).replace(regexBeatsStyle, target);
  });
  return pipeline;
}

function mutatePipelineContentWithNewProcessor(jsonPipelineContent: any, processor: any) {
  if (!jsonPipelineContent.processors) {
    jsonPipelineContent.processors = [];
  }

  jsonPipelineContent.processors.push(processor);
}

export function addCustomPipelineAndLocalRoutingRulesProcessor(
  pipeline: PipelineInstall
): PipelineInstall {
  if (!pipeline.shouldInstallCustomPipelines || !pipeline.dataStream) {
    return pipeline;
  }

  const localRoutingRules =
    pipeline.dataStream?.routing_rules?.find(
      (rule) => rule.source_dataset === pipeline.dataStream?.dataset
    )?.rules ?? [];

  const customPipelineProcessors = [
    {
      pipeline: {
        name: 'global@custom',
        ignore_missing_pipeline: true,
        description: '[Fleet] Global pipeline for all data streams',
      },
    },
    {
      pipeline: {
        name: `${pipeline.dataStream.type}@custom`,
        ignore_missing_pipeline: true,
        description: `[Fleet] Pipeline for all data streams of type \`${pipeline.dataStream.type}\``,
      },
    },
    ...(pipeline.dataStream.package
      ? [
          {
            pipeline: {
              // This pipeline name gets the `.integration` suffix to avoid conflicts with the pipeline name for the dataset below
              name: `${pipeline.dataStream.type}-${pipeline.dataStream.package}.integration@custom`,
              ignore_missing_pipeline: true,
              description: `[Fleet] Pipeline for all data streams of type \`${pipeline.dataStream.type}\` defined by the \`${pipeline.dataStream.package}\` integration`,
            },
          },
        ]
      : []),
    {
      pipeline: {
        name: `${pipeline.dataStream.type}-${pipeline.dataStream.dataset}@custom`,
        ignore_missing_pipeline: true,
        description: `[Fleet] Pipeline for the \`${pipeline.dataStream.dataset}\` dataset`,
      },
    },
  ];

  const rerouteProcessors = localRoutingRules.map((routingRule) => ({
    reroute: {
      tag: pipeline.dataStream?.dataset,
      dataset: routingRule.target_dataset,
      namespace: routingRule.namespace,
      if: routingRule.if,
    },
  }));

  if (pipeline.extension === 'yml') {
    const parsedPipelineContent = load(pipeline.contentForInstallation);
    customPipelineProcessors.forEach((processor) =>
      mutatePipelineContentWithNewProcessor(parsedPipelineContent, processor)
    );
    rerouteProcessors.forEach((processor) =>
      mutatePipelineContentWithNewProcessor(parsedPipelineContent, processor)
    );
    return {
      ...pipeline,
      contentForInstallation: `---\n${dump(parsedPipelineContent)}`,
    };
  }

  const parsedPipelineContent = JSON.parse(pipeline.contentForInstallation);
  customPipelineProcessors.forEach((processor) =>
    mutatePipelineContentWithNewProcessor(parsedPipelineContent, processor)
  );
  rerouteProcessors.forEach((processor) =>
    mutatePipelineContentWithNewProcessor(parsedPipelineContent, processor)
  );

  return {
    ...pipeline,
    contentForInstallation: JSON.stringify(parsedPipelineContent),
  };
}
