/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

// eslint-disable-next-line import/order
import { createAppContextStartContractMock } from '../../../../mocks';

jest.mock('../../packages/get', () => {
  return { getInstallation: jest.fn(), getInstallationObject: jest.fn() };
});

import { errors } from '@elastic/elasticsearch';
import type { SavedObject, SavedObjectsClientContract } from '@kbn/core/server';
import { loggerMock } from '@kbn/logging-mocks';

import { savedObjectsClientMock } from '@kbn/core/server/mocks';
import { elasticsearchClientMock } from '@kbn/core-elasticsearch-client-server-mocks';

import { getInstallation, getInstallationObject } from '../../packages';
import type { Installation } from '../../../../types';
import { ElasticsearchAssetType } from '../../../../types';
import { appContextService } from '../../../app_context';

import { getESAssetMetadata } from '../meta';

import { createArchiveIteratorFromMap } from '../../archive/archive_iterator';
import type { PackageInstallContext } from '../../../../../common/types';
import { PACKAGES_SAVED_OBJECT_TYPE } from '../../../../constants';

import { installTransforms } from './install';

describe('test transform install with legacy schema', () => {
  let esClient: ReturnType<typeof elasticsearchClientMock.createElasticsearchClient>;
  let savedObjectsClient: jest.Mocked<SavedObjectsClientContract>;
  beforeEach(() => {
    appContextService.start(createAppContextStartContractMock());
    esClient = elasticsearchClientMock.createClusterClient().asInternalUser;
    (getInstallation as jest.MockedFunction<typeof getInstallation>).mockReset();
    (getInstallationObject as jest.MockedFunction<typeof getInstallationObject>).mockReset();
    savedObjectsClient = savedObjectsClientMock.create();
    savedObjectsClient.update.mockImplementation(async (type, id, attributes) => ({
      type: PACKAGES_SAVED_OBJECT_TYPE,
      id: 'endpoint',
      attributes,
      references: [],
    }));
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  test('can install new versions and removes older version', async () => {
    const previousInstallation: Installation = {
      installed_es: [
        {
          id: 'metrics-endpoint.policy-0.16.0-dev.0',
          type: ElasticsearchAssetType.ingestPipeline,
        },
        {
          id: 'endpoint.metadata_current-default-0.15.0-dev.0',
          type: ElasticsearchAssetType.transform,
        },
      ],
    } as unknown as Installation;

    const currentInstallation: Installation = {
      installed_es: [
        {
          id: 'metrics-endpoint.policy-0.16.0-dev.0',
          type: ElasticsearchAssetType.ingestPipeline,
        },
        {
          id: 'endpoint.metadata_current-default-0.15.0-dev.0',
          type: ElasticsearchAssetType.transform,
        },
        {
          id: 'endpoint.metadata_current-default-0.16.0-dev.0',
          type: ElasticsearchAssetType.transform,
        },
        {
          id: 'endpoint.metadata-default-0.16.0-dev.0',
          type: ElasticsearchAssetType.transform,
        },
      ],
    } as unknown as Installation;
    (getInstallation as jest.MockedFunction<typeof getInstallation>)
      .mockReturnValueOnce(Promise.resolve(previousInstallation))
      .mockReturnValueOnce(Promise.resolve(currentInstallation));

    (
      getInstallationObject as jest.MockedFunction<typeof getInstallationObject>
    ).mockReturnValueOnce(
      Promise.resolve({
        attributes: {
          installed_es: previousInstallation.installed_es,
        },
      } as unknown as SavedObject<Installation>)
    );

    esClient.transform.getTransform.mockResponseOnce({
      count: 1,
      transforms: [
        // @ts-expect-error incomplete data
        {
          dest: {
            index: 'index',
          },
        },
      ],
    });

    await installTransforms({
      packageInstallContext: {
        packageInfo: {
          name: 'endpoint',
          version: '0.16.0-dev.0',
          data_streams: [
            {
              type: 'metrics',
              dataset: 'endpoint.metadata',
              title: 'Endpoint Metadata',
              release: 'experimental',
              package: 'endpoint',
              ingest_pipeline: 'default',
              elasticsearch: {
                'index_template.mappings': {
                  dynamic: false,
                },
              },
              path: 'metadata',
            },
            {
              type: 'metrics',
              dataset: 'endpoint.metadata_current',
              title: 'Endpoint Metadata Current',
              release: 'experimental',
              package: 'endpoint',
              ingest_pipeline: 'default',
              elasticsearch: {
                'index_template.mappings': {
                  dynamic: false,
                },
              },
              path: 'metadata_current',
            },
          ],
        },
        paths: [
          'endpoint-0.16.0-dev.0/data_stream/policy/elasticsearch/ingest_pipeline/default.json',
          'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata/default.json',
          'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json',
        ],
        assetsMap: new Map([
          [
            'endpoint-0.16.0-dev.0/data_stream/policy/elasticsearch/ingest_pipeline/default.json',
            Buffer.from('{"content": "data"}'),
          ],
          [
            'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata/default.json',
            Buffer.from('{"content": "data"}'),
          ],
          [
            'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json',
            Buffer.from('{"content": "data"}'),
          ],
        ]),
        archiveIterator: createArchiveIteratorFromMap(
          new Map([
            [
              'endpoint-0.16.0-dev.0/data_stream/policy/elasticsearch/ingest_pipeline/default.json',
              Buffer.from('{"content": "data"}'),
            ],
            [
              'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata/default.json',
              Buffer.from('{"content": "data"}'),
            ],
            [
              'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json',
              Buffer.from('{"content": "data"}'),
            ],
          ])
        ),
      } as unknown as PackageInstallContext,
      esClient,
      savedObjectsClient,
      logger: loggerMock.create(),
      esReferences: previousInstallation.installed_es,
    });

    expect(esClient.transform.stopTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata_current-default-0.15.0-dev.0',
          force: true,
        },
        { ignore: [404] },
      ],
    ]);
    expect(esClient.transform.deleteTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata_current-default-0.15.0-dev.0',
          force: true,
          delete_dest_index: true,
        },
        { ignore: [404] },
      ],
    ]);

    const meta = getESAssetMetadata({ packageName: 'endpoint' });

    expect(esClient.transform.putTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata-default-0.16.0-dev.0',
          defer_validation: true,
          content: 'data',
          _meta: meta,
        },
        { ignore: [409] },
      ],
      [
        {
          transform_id: 'endpoint.metadata_current-default-0.16.0-dev.0',
          defer_validation: true,
          content: 'data',
          _meta: meta,
        },
        { ignore: [409] },
      ],
    ]);
    expect(esClient.transform.startTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata-default-0.16.0-dev.0',
        },
        { ignore: [409] },
      ],
      [
        {
          transform_id: 'endpoint.metadata_current-default-0.16.0-dev.0',
        },
        { ignore: [409] },
      ],
    ]);

    expect(savedObjectsClient.update.mock.calls).toEqual([
      [
        'epm-packages',
        'endpoint',
        {
          installed_es: [
            {
              id: 'metrics-endpoint.policy-0.16.0-dev.0',
              type: 'ingest_pipeline',
            },
            {
              id: 'endpoint.metadata_current-default-0.15.0-dev.0',
              type: 'transform',
            },
            {
              id: 'endpoint.metadata-default-0.16.0-dev.0',
              type: 'transform',
            },
            {
              id: 'endpoint.metadata_current-default-0.16.0-dev.0',
              type: 'transform',
            },
          ],
        },
        {
          refresh: false,
        },
      ],
      [
        'epm-packages',
        'endpoint',
        {
          installed_es: [
            {
              id: 'metrics-endpoint.policy-0.16.0-dev.0',
              type: 'ingest_pipeline',
            },
            {
              id: 'endpoint.metadata-default-0.16.0-dev.0',
              type: 'transform',
            },
            {
              id: 'endpoint.metadata_current-default-0.16.0-dev.0',
              type: 'transform',
            },
          ],
        },
        {
          refresh: false,
        },
      ],
    ]);
  });

  test('can install new version and when no older version', async () => {
    const previousInstallation: Installation = {
      installed_es: [],
    } as unknown as Installation;

    const currentInstallation: Installation = {
      installed_es: [
        {
          id: 'metrics-endpoint.metadata-current-default-0.16.0-dev.0',
          type: ElasticsearchAssetType.transform,
        },
      ],
    } as unknown as Installation;
    (getInstallation as jest.MockedFunction<typeof getInstallation>)
      .mockReturnValueOnce(Promise.resolve(previousInstallation))
      .mockReturnValueOnce(Promise.resolve(currentInstallation));

    (
      getInstallationObject as jest.MockedFunction<typeof getInstallationObject>
    ).mockReturnValueOnce(
      Promise.resolve({
        attributes: { installed_es: [] },
      } as unknown as SavedObject<Installation>)
    );

    await installTransforms({
      packageInstallContext: {
        packageInfo: {
          name: 'endpoint',
          version: '0.16.0-dev.0',
          data_streams: [
            {
              type: 'metrics',
              dataset: 'endpoint.metadata_current',
              title: 'Endpoint Metadata',
              release: 'experimental',
              package: 'endpoint',
              ingest_pipeline: 'default',
              elasticsearch: {
                'index_template.mappings': {
                  dynamic: false,
                },
              },
              path: 'metadata_current',
            },
          ],
        },
        paths: ['endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json'],
        assetsMap: new Map([
          [
            'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json',
            Buffer.from('{"content": "data"}'),
          ],
        ]),
        archiveIterator: createArchiveIteratorFromMap(
          new Map([
            [
              'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json',
              Buffer.from('{"content": "data"}'),
            ],
          ])
        ),
      } as unknown as PackageInstallContext,
      esClient,
      savedObjectsClient,
      logger: loggerMock.create(),
      esReferences: previousInstallation.installed_es,
    });

    const meta = getESAssetMetadata({ packageName: 'endpoint' });

    expect(esClient.transform.putTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata_current-default-0.16.0-dev.0',
          defer_validation: true,
          content: 'data',
          _meta: meta,
        },
        { ignore: [409] },
      ],
    ]);
    expect(esClient.transform.startTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata_current-default-0.16.0-dev.0',
        },
        { ignore: [409] },
      ],
    ]);

    expect(savedObjectsClient.update.mock.calls).toEqual([
      [
        'epm-packages',
        'endpoint',
        {
          installed_es: [
            { id: 'endpoint.metadata_current-default-0.16.0-dev.0', type: 'transform' },
          ],
        },
        {
          refresh: false,
        },
      ],
    ]);
  });

  test('can removes older version when no new install in package', async () => {
    const previousInstallation: Installation = {
      installed_es: [
        {
          id: 'endpoint.metadata-current-default-0.15.0-dev.0',
          type: ElasticsearchAssetType.transform,
        },
      ],
    } as unknown as Installation;

    const currentInstallation: Installation = {
      installed_es: [],
    } as unknown as Installation;

    (getInstallation as jest.MockedFunction<typeof getInstallation>)
      .mockReturnValueOnce(Promise.resolve(previousInstallation))
      .mockReturnValueOnce(Promise.resolve(currentInstallation));

    (
      getInstallationObject as jest.MockedFunction<typeof getInstallationObject>
    ).mockReturnValueOnce(
      Promise.resolve({
        attributes: { installed_es: currentInstallation.installed_es },
      } as unknown as SavedObject<Installation>)
    );

    esClient.transform.getTransform.mockResponseOnce({
      count: 1,
      transforms: [
        // @ts-expect-error incomplete data
        {
          dest: {
            index: 'index',
          },
        },
      ],
    });

    await installTransforms({
      packageInstallContext: {
        packageInfo: {
          name: 'endpoint',
          version: '0.16.0-dev.0',
          data_streams: [
            {
              type: 'metrics',
              dataset: 'endpoint.metadata',
              title: 'Endpoint Metadata',
              release: 'experimental',
              package: 'endpoint',
              ingest_pipeline: 'default',
              elasticsearch: {
                'index_template.mappings': {
                  dynamic: false,
                },
              },
              path: 'metadata',
            },
            {
              type: 'metrics',
              dataset: 'endpoint.metadata_current',
              title: 'Endpoint Metadata Current',
              release: 'experimental',
              package: 'endpoint',
              ingest_pipeline: 'default',
              elasticsearch: {
                'index_template.mappings': {
                  dynamic: false,
                },
              },
              path: 'metadata_current',
            },
          ],
        },
        paths: [],
      } as unknown as PackageInstallContext,
      esClient,
      savedObjectsClient,
      logger: loggerMock.create(),
      esReferences: previousInstallation.installed_es,
    });

    expect(esClient.transform.stopTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata-current-default-0.15.0-dev.0',
          force: true,
        },
        { ignore: [404] },
      ],
    ]);

    expect(esClient.transform.deleteTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata-current-default-0.15.0-dev.0',
          force: true,
          delete_dest_index: true,
        },
        { ignore: [404] },
      ],
    ]);

    expect(savedObjectsClient.update.mock.calls).toEqual([
      [
        'epm-packages',
        'endpoint',
        {
          installed_es: [],
        },
        {
          refresh: false,
        },
      ],
    ]);
  });

  test('ignore already exists error if saved object and ES transforms are out of sync', async () => {
    const previousInstallation: Installation = {
      installed_es: [],
    } as unknown as Installation;

    const currentInstallation: Installation = {
      installed_es: [
        {
          id: 'metrics-endpoint.metadata-current-default-0.16.0-dev.0',
          type: ElasticsearchAssetType.transform,
        },
      ],
    } as unknown as Installation;
    (getInstallation as jest.MockedFunction<typeof getInstallation>)
      .mockReturnValueOnce(Promise.resolve(previousInstallation))
      .mockReturnValueOnce(Promise.resolve(currentInstallation));

    (
      getInstallationObject as jest.MockedFunction<typeof getInstallationObject>
    ).mockReturnValueOnce(
      Promise.resolve({
        attributes: { installed_es: [] },
      } as unknown as SavedObject<Installation>)
    );

    esClient.transport.request.mockImplementationOnce(() =>
      elasticsearchClientMock.createErrorTransportRequestPromise(
        new errors.ResponseError(
          elasticsearchClientMock.createApiResponse({
            statusCode: 400,
            body: { error: { type: 'resource_already_exists_exception' } },
          })
        )
      )
    );

    await installTransforms({
      packageInstallContext: {
        packageInfo: {
          name: 'endpoint',
          version: '0.16.0-dev.0',
          data_streams: [
            {
              type: 'metrics',
              dataset: 'endpoint.metadata_current',
              title: 'Endpoint Metadata',
              release: 'experimental',
              package: 'endpoint',
              ingest_pipeline: 'default',
              elasticsearch: {
                'index_template.mappings': {
                  dynamic: false,
                },
              },
              path: 'metadata_current',
            },
          ],
        },
        paths: ['endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json'],
        assetsMap: new Map([
          [
            'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json',
            Buffer.from('{"content": "data"}'),
          ],
        ]),
        archiveIterator: createArchiveIteratorFromMap(
          new Map([
            [
              'endpoint-0.16.0-dev.0/elasticsearch/transform/metadata_current/default.json',
              Buffer.from('{"content": "data"}'),
            ],
          ])
        ),
      } as unknown as PackageInstallContext,
      esClient,
      savedObjectsClient,
      logger: loggerMock.create(),
      esReferences: previousInstallation.installed_es,
    });

    const meta = getESAssetMetadata({ packageName: 'endpoint' });

    expect(esClient.transform.putTransform.mock.calls).toEqual([
      [
        {
          transform_id: 'endpoint.metadata_current-default-0.16.0-dev.0',
          defer_validation: true,
          content: 'data',
          _meta: meta,
        },
        { ignore: [409] },
      ],
    ]);
    expect(esClient.transform.startTransform.mock.calls).toEqual([
      [{ transform_id: 'endpoint.metadata_current-default-0.16.0-dev.0' }, { ignore: [409] }],
    ]);

    expect(savedObjectsClient.update.mock.calls).toEqual([
      [
        'epm-packages',
        'endpoint',
        {
          installed_es: [
            { id: 'endpoint.metadata_current-default-0.16.0-dev.0', type: 'transform' },
          ],
        },
        {
          refresh: false,
        },
      ],
    ]);
  });
});
