/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { i18n } from '@kbn/i18n';
import React, { useCallback } from 'react';
import { FormattedMessage } from '@kbn/i18n-react';
import {
  EuiFlexGroup,
  EuiFlexItem,
  EuiPanel,
  EuiEmptyPrompt,
  EuiTitle,
  EuiSelect,
  EuiLoadingChart,
  useEuiTheme,
} from '@elastic/eui';
import {
  Axis,
  BarSeries,
  Chart,
  CurveType,
  LineSeries,
  Settings,
  LEGACY_LIGHT_THEME,
} from '@elastic/charts';
import { assign, fill } from 'lodash';
import moment from 'moment';
import { formatMillisForDisplay } from '../../../lib/execution_duration_utils';

export interface ComponentOpts {
  executionDuration: {
    average: number;
    valuesWithTimestamp: Record<string, number>;
  };
  numberOfExecutions: number;
  onChangeDuration: (length: number) => void;
  isLoading?: boolean;
}

const NUM_EXECUTIONS_OPTIONS = [120, 60, 30, 15].map((value) => ({
  value,
  text: i18n.translate(
    'xpack.triggersActionsUI.sections.executionDurationChart.numberOfExecutionsOption',
    {
      defaultMessage: '{value} runs',
      values: {
        value,
      },
    }
  ),
}));

export const ExecutionDurationChart: React.FunctionComponent<ComponentOpts> = ({
  executionDuration,
  numberOfExecutions,
  onChangeDuration,
  isLoading,
}: ComponentOpts) => {
  const { euiTheme } = useEuiTheme();

  const paddedExecutionDurations = padOrTruncateDurations(
    executionDuration.valuesWithTimestamp,
    numberOfExecutions
  );

  const onChange: React.ChangeEventHandler<HTMLSelectElement> = useCallback(
    ({ target }) => onChangeDuration(Number(target.value)),
    [onChangeDuration]
  );

  return (
    <EuiPanel data-test-subj="executionDurationChartPanel" hasBorder={true}>
      <EuiFlexGroup alignItems="center" gutterSize="xs">
        <EuiFlexItem grow={false}>
          <EuiTitle size="xxs">
            <h4>
              <FormattedMessage
                id="xpack.triggersActionsUI.sections.executionDurationChart.recentDurationsTitle"
                defaultMessage="Recent run durations"
              />
            </h4>
          </EuiTitle>
        </EuiFlexItem>
        <EuiFlexGroup justifyContent="flexEnd">
          <EuiFlexItem grow={false}>
            <EuiSelect
              id="select-number-execution-durations"
              data-test-subj="executionDurationChartPanelSelect"
              options={NUM_EXECUTIONS_OPTIONS}
              value={numberOfExecutions}
              aria-label={i18n.translate(
                'xpack.triggersActionsUI.sections.executionDurationChart.selectNumberOfExecutionDurationsLabel',
                {
                  defaultMessage: 'Select number of runs',
                }
              )}
              onChange={onChange}
            />
          </EuiFlexItem>
        </EuiFlexGroup>
      </EuiFlexGroup>
      {isLoading && (
        <EuiFlexGroup justifyContent="center">
          <EuiFlexItem grow={false} style={{ height: '80px', justifyContent: 'center' }}>
            <EuiLoadingChart size="xl" />
          </EuiFlexItem>
        </EuiFlexGroup>
      )}
      {!isLoading &&
        (executionDuration.valuesWithTimestamp &&
        Object.entries(executionDuration.valuesWithTimestamp).length > 0 ? (
          <>
            <Chart data-test-subj="executionDurationChart" size={{ height: 80 }}>
              <Settings
                theme={{
                  lineSeriesStyle: {
                    point: { visible: 'never' },
                    line: { stroke: euiTheme.colors.accent },
                  },
                }}
                // TODO connect to charts.theme service see src/plugins/charts/public/services/theme/README.md
                baseTheme={LEGACY_LIGHT_THEME}
                locale={i18n.getLocale()}
              />
              <BarSeries
                id="executionDuration"
                name={i18n.translate(
                  'xpack.triggersActionsUI.sections.executionDurationChart.durationLabel',
                  {
                    defaultMessage: `Duration`,
                  }
                )}
                xScaleType="linear"
                yScaleType="linear"
                xAccessor={0}
                yAccessors={[1]}
                data={paddedExecutionDurations.map(([timestamp, val], ndx) => [
                  timestamp ? moment(timestamp).format('D MMM YYYY @ HH:mm:ss') : ndx,
                  val,
                ])}
                minBarHeight={2}
              />
              <LineSeries
                id="rule_duration_avg"
                name={i18n.translate(
                  'xpack.triggersActionsUI.sections.executionDurationChart.avgDurationLabel',
                  {
                    defaultMessage: `Avg Duration`,
                  }
                )}
                xScaleType="linear"
                yScaleType="linear"
                xAccessor={0}
                yAccessors={[1]}
                data={paddedExecutionDurations.map(([timestamp, val], ndx) => [
                  timestamp ? moment(timestamp).format('D MMM YYYY @ HH:mm:ss') : ndx,
                  val ? executionDuration.average : null,
                ])}
                curve={CurveType.CURVE_NATURAL}
              />
              <Axis id="left-axis" position="left" tickFormat={(d) => formatMillisForDisplay(d)} />
            </Chart>
          </>
        ) : (
          <>
            <EuiEmptyPrompt
              data-test-subj="executionDurationChartEmpty"
              body={
                <>
                  <p>
                    <FormattedMessage
                      id="xpack.triggersActionsUI.sections.executionDurationChart.executionDurationNoData"
                      defaultMessage="There is no available run duration information for this rule."
                    />
                  </p>
                </>
              }
            />
          </>
        ))}
    </EuiPanel>
  );
};

export function padOrTruncateDurations(
  valuesWithTimestamp: Record<string, number>,
  desiredSize: number
) {
  const values = Object.entries(valuesWithTimestamp);
  if (values.length === desiredSize) {
    return values;
  } else if (values.length < desiredSize) {
    return assign(fill(new Array(desiredSize), [null, null]), values);
  } else {
    // oldest durations are at the start of the array, so take the last {desiredSize} values
    return values.slice(-desiredSize);
  }
}
